/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServerConfiguration.java msauvage $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.webadmin.bo;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.management.AttributeNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;

import org.objectweb.petals.kernel.admin.PetalsAdminServiceMBean;
import org.objectweb.petals.tools.webadmin.util.JMXUtils;

/**
 * Represents the configuration of a server as a HashMap of his properties :
 * host, jmxPort, jmxLogin, jmxPassword and with an attribute : a
 * MBeanServerConnection connection to the server
 */
public class ServerConfiguration {

    private static final String MONITOR = "monitor";

    private static final String SHOW_MESSAGE_CONTENT = "showMessageContent";

    private static final String STATE = "state";

    private static final String JMX_PASSWORD = "jmxPassword";

    private static final String JMX_LOGIN = "jmxLogin";

    private static final String JMX_PORT = "jmxPort";

    private static final String HOST = "host";

    private static final String NAME = "name";

    /**
     * 
     */
    private static final long serialVersionUID = -7879504327924775052L;

    /**
     * connection to the server
     */
    private MBeanServerConnection connection;

    /**
     * the serv's informations
     */
    private Map<String, String> serverProperties;

    /**
     * Creates a new empty ServerConfiguration
     * 
     */
    public ServerConfiguration() {
        serverProperties = new HashMap<String, String>();
    }

    /**
     * Creatse a new ServerConfiguration, based on a HashMap containing the
     * connexion's informations
     * 
     * @param serverInfos
     *            HashMap containing the infos
     */
    public ServerConfiguration(Map<String, String> serverInfos) {
        serverProperties = new HashMap<String, String>();
        serverProperties.put(NAME, serverInfos.get(NAME));
        serverProperties.put(HOST, serverInfos.get(HOST));
        serverProperties.put(JMX_PORT, serverInfos.get(JMX_PORT));
        serverProperties.put(JMX_LOGIN, serverInfos.get(JMX_LOGIN));
        serverProperties.put(JMX_PASSWORD, serverInfos.get(JMX_PASSWORD));
        serverProperties.put(STATE, serverInfos.get(STATE));
        serverProperties.put(MONITOR, serverInfos.get(MONITOR));
        serverProperties.put(SHOW_MESSAGE_CONTENT, serverInfos
            .get(SHOW_MESSAGE_CONTENT));
        if (STATE.equalsIgnoreCase(PetalsAdminServiceMBean.STATE_STARTED)) {
            try {
                this.createConnection();
            } catch (IOException e) {
                // exception ignore : connection not created
                e.printStackTrace(System.err);
            }
        }
    }

    /**
     * Creates a new ServerConfiguration with base connection's informations
     * 
     * @param host
     *            host of the server
     * @param jmxPort
     *            JMX port of the server
     * @param jmxLogin
     *            JMX login of the server
     * @param jmxPassword
     *            JMX password of the server
     */
    public ServerConfiguration(String host, String jmxPort, String jmxLogin,
        String jmxPassword) {
        serverProperties = new HashMap<String, String>();
        serverProperties.put(HOST, host);
        serverProperties.put(JMX_PORT, jmxPort);
        serverProperties.put(JMX_LOGIN, jmxLogin);
        serverProperties.put(JMX_PASSWORD, jmxPassword);
    }

    /**
     * Creates a new ServerConfiguration with base informations
     * 
     * @param name
     *            the name of the server
     * @param host
     *            host of the server
     * @param jmxPort
     *            JMX port of the server
     * @param jmxLogin
     *            JMX login of the server
     * @param jmxPassword
     *            JMX password of the server
     */
    public ServerConfiguration(String name, String host, String jmxPort,
        String jmxLogin, String jmxPassword) {
        serverProperties = new HashMap<String, String>();
        serverProperties.put(NAME, name);
        serverProperties.put(HOST, host);
        serverProperties.put(JMX_PORT, jmxPort);
        serverProperties.put(JMX_LOGIN, jmxLogin);
        serverProperties.put(JMX_PASSWORD, jmxPassword);
    }

    /**
     * Creates a new ServerConfiguration with base informations
     * 
     * @param name
     *            the name of the server
     * @param host
     *            host of the server
     * @param jmxPort
     *            JMX port of the server
     * @param jmxLogin
     *            JMX login of the server
     * @param jmxPassword
     *            JMX password of the server
     * @param state
     *            the state of the server (started, stopped)
     */
    public ServerConfiguration(String name, String host, String jmxPort,
        String jmxLogin, String jmxPassword, String state) {
        serverProperties = new HashMap<String, String>();
        serverProperties.put(NAME, name);
        serverProperties.put(HOST, host);
        serverProperties.put(JMX_PORT, jmxPort);
        serverProperties.put(JMX_LOGIN, jmxLogin);
        serverProperties.put(JMX_PASSWORD, jmxPassword);
        serverProperties.put(STATE, state);
    }

    /**
     * Creates a new ServerConfiguration with complete informations
     * 
     * @param name
     *            the name of the server
     * @param host
     *            host of the server
     * @param jmxPort
     *            JMX port of the server
     * @param jmxLogin
     *            JMX login of the server
     * @param jmxPassword
     *            JMX password of the server
     * @param state
     *            the state of the server (started, stopped)
     * @param connection
     *            a MBeanServerConnection connection to the server
     */
    public ServerConfiguration(String name, String host, String jmxPort,
        String jmxLogin, String jmxPassword, String state,
        MBeanServerConnection connection) {
        serverProperties = new HashMap<String, String>();
        serverProperties.put(NAME, name);
        serverProperties.put(HOST, host);
        serverProperties.put(JMX_PORT, jmxPort);
        serverProperties.put(JMX_LOGIN, jmxLogin);
        serverProperties.put(JMX_PASSWORD, jmxPassword);
        serverProperties.put(STATE, state);
        this.connection = connection;
    }

    /**
     * Retrieves a new connection to the server and sets the connection
     * attribute
     * 
     * @throws IOException
     * 
     */
    public void createConnection() throws IOException {
        try {
            this.connection = JMXUtils.getJMXUtils().retrieveConnector(this)
                .getMBeanServerConnection();
        } catch (IOException e) {
            this.connection = null;
            throw new IOException("JMX server unreachable at address "
                + this.getAddress());
        }
    }

    public String getAddress() {
        return "'" + getJmxLogin() + "@" + this.getHost() + ":"
            + this.getJmxPort() + "'";
    }

    /**
     * Returns the connection to the server
     * 
     * @return a MBeanServerConnection connection
     */
    public MBeanServerConnection getConnection() {
        return this.connection;
    }

    /**
     * Gets basics server's informations for connection : name, host, port,
     * login, password
     * 
     * @return a Map containing informations as <key,value> pairs
     */
    public Map<String, String> getConnectionParameters() {
        Map<String, String> serverInfos = new HashMap<String, String>();
        serverInfos.put(NAME, this.getName());
        serverInfos.put(HOST, this.getHost());
        serverInfos.put(JMX_PORT, this.getJmxPort());
        serverInfos.put(JMX_LOGIN, this.getJmxLogin());
        serverInfos.put(JMX_PASSWORD, this.getJmxPassword());
        return serverInfos;
    }

    /**
     * Returns the host of the server
     * 
     * @return a String representation fo the host
     */
    public String getHost() {
        return serverProperties.get(HOST);
    }

    /**
     * Returns the JMX login name
     * 
     * @return a string representation of the JMX login name
     */
    public String getJmxLogin() {
        return serverProperties.get(JMX_LOGIN);
    }

    /**
     * Returns the JMX user password
     * 
     * @return the JMX password as a String
     */
    public String getJmxPassword() {
        return serverProperties.get(JMX_PASSWORD);
    }

    /**
     * Returns the JMX port
     * 
     * @return a string representation of the port
     */
    public String getJmxPort() {
        return serverProperties.get(JMX_PORT);
    }

    public String getShowMessageContent() {
        return serverProperties.get(SHOW_MESSAGE_CONTENT);
    }

    public String getMonitor() {
        return serverProperties.get(MONITOR);
    }

    /**
     * Returns the name of the server
     * 
     * @return the name of the server as a String
     */
    public String getName() {
        return serverProperties.get(NAME);
    }

    /**
     * Gets the server's informations as a <name,value> set of parameters
     * 
     * @return a Map representation of the connection properties
     */
    public Map<String, String> getServerProperties() {
        return serverProperties;
    }

    /**
     * Returns the state of the server
     * 
     * @return a string representation of the state of the server
     */
    public String getState() {
        return serverProperties.get(STATE);
    }

    /**
     * Sets the connection to the server
     * 
     * @param state
     *            a MBeanServerConnection that is to be the connection
     */
    public void setConnection(MBeanServerConnection connection) {
        this.connection = connection;
    }

    /**
     * Sets the host of the server
     * 
     * @param host
     *            a String that is to be the host of the server
     */
    public void setHost(String host) {
        serverProperties.put(HOST, host);
    }

    /**
     * Sets the JMX login name
     * 
     * @param jmxLogin
     *            a String that is to be the JMX login name
     */
    public void setJmxLogin(String jmxLogin) {
        serverProperties.put(JMX_LOGIN, jmxLogin);
    }

    /**
     * Sets the JMX user password
     * 
     * @param jmxPassword
     *            a String that is to be password for jmx access
     */
    public void setJmxPassword(String jmxPassword) {
        serverProperties.put(JMX_PASSWORD, jmxPassword);
    }

    /**
     * Sets the JMX port
     * 
     * @param jmxPort
     *            a String that is to be the port
     */
    public void setJmxPort(String jmxPort) {
        serverProperties.put(JMX_PORT, jmxPort);
    }

    public void setMonitor(boolean monitor) {
        String state = PetalsAdminServiceMBean.STATE_STOPPED;
        if (monitor) {
            state = PetalsAdminServiceMBean.STATE_STARTED;
        }
        serverProperties.put(MONITOR, state);
    }

    /**
     * Retrieve the name of the server, and sets it
     * 
     * @throws IOException
     * @throws MBeanException
     * @throws AttributeNotFoundException 
     * 
     * @throws Exception
     */
    public void setName() throws MBeanException, IOException, AttributeNotFoundException {
        serverProperties.put(NAME, JMXUtils.getJMXUtils().getServerName(this));
    }

    /**
     * Sets the name of the server
     * 
     * @param name
     *            a String that is to be the server's name
     */
    public void setName(String name) {
        serverProperties.put(NAME, name);
    }

    /**
     * Sets the new connection properties for the server
     * 
     * @param connectionProperties
     */
    public void setServerProperties(Map<String, String> connectionProperties) {
        this.serverProperties = connectionProperties;
    }

    public void setShowMessageContent(boolean showMessageContent) {
        serverProperties.put(SHOW_MESSAGE_CONTENT, new Boolean(
            showMessageContent).toString());
    }

    /**
     * Sets the state of the server
     * 
     * @param state
     *            a String that is to be the state
     */
    public void setState(String state) {
        serverProperties.put(STATE, state);
    }

    public String toString() {
        return serverProperties.toString();
    }

}
