/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MsgsPerSecondDataProd.java msauvage $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.webadmin.cewolf.dataproducers;

import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.management.MBeanException;

import org.jfree.data.time.Second;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.objectweb.petals.tools.webadmin.service.NewJMXService;
import org.objectweb.petals.tools.webadmin.util.MathTools;

import de.laures.cewolf.DatasetProduceException;
import de.laures.cewolf.DatasetProducer;

public final class MsgsPerSecondDataProd implements DatasetProducer {

    private static final int MINUTE_IN_MILLIS = 59000;

    /**
     * 
     */
    private static final long serialVersionUID = -1424547051372969763L;

    public MsgsPerSecondDataProd() {
        super();
    }

    /**
     * Returns the id of the producer
     * 
     * @return an id as a String
     */
    public String getProducerId() {
        return new String("MsgsPerSecondDataset Producer");
    }

    /**
     * Indicates if dataset produced has expired. method is called by the Cewolf
     * framework to check if a formerly produced data can be reused. As datas
     * are evolving and retrieved in an MBean, method always returns true to
     * force reloading datas.
     * 
     * @return true if the data which had been produced with the passed in
     *         parameters has expired since its creation, false otherwise
     */
    public boolean hasExpired(Map params, Date since) {
        return true;
    }

    @SuppressWarnings("unchecked")
    public Object produceDataset(Map params) throws DatasetProduceException {
        // retrieve the jmx service
        NewJMXService jmxService = NewJMXService.getJmxService();
        // getting parameters
        String seriesTitle = (String) params.get("seriesTitle");
        String mbeanName = (String) params.get("mbeanName");
        String mbeanType = (String) params.get("mbeanType");
        String datasetName = (String) params.get("datasetName");
        long sinceTime = ((Long) params.get("sinceTime")).longValue();

        Map<Long, Long> msgPerSecond = new HashMap<Long, Long>();

        // creating period timestamps
        long endTime = sinceTime - 5000 + MINUTE_IN_MILLIS;

        try {
            msgPerSecond = jmxService.getTimeDatas(mbeanName, mbeanType,
                datasetName, sinceTime);
        } catch (IOException e) {
            e.printStackTrace();
            throw new DatasetProduceException(e.getMessage());
        } catch (MBeanException e) {
            e.printStackTrace();
            throw new DatasetProduceException(e.getMessage());
        }

        // dataset containing one or more serie(s) of time datas
        TimeSeriesCollection dataset = new TimeSeriesCollection();

        // receive a series of time datas
        TimeSeries timeDatas = new TimeSeries(seriesTitle, Second.class);
        // calendar used to format dates
        Calendar cal = Calendar.getInstance();

        if (msgPerSecond != null && msgPerSecond.size() > 0) {
            for (long timestamp = sinceTime; timestamp <= endTime; 
            timestamp += MathTools.THOUSAND) {
                // rounding timestamp to second
                long roundedTimestamp = (timestamp / MathTools.THOUSAND)
                    * MathTools.THOUSAND;
                // formatting timestamp to a Date with the calendar
                Date currentTime = new Date(roundedTimestamp);
                cal.setTime(currentTime);
                // initializing count to zero, if there is a number associated
                // to it in the datas retrieved, we get it, else count will be 0
                long currentCount = 0L;
                if (msgPerSecond.get(roundedTimestamp) != null) {
                    currentCount = msgPerSecond.get(roundedTimestamp);
                }
                // adding datas to series
                timeDatas.add(new Second(currentTime), currentCount);
            }
            // adding series to dataset
            dataset.addSeries(timeDatas);
        }
        return dataset;
    }

}
