/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DataCollector.java msauvage $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.tools.webadmin.collector;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.objectweb.petals.kernel.admin.PetalsAdminServiceMBean;
import org.objectweb.petals.tools.jmx.api.PetalsDeploymentServiceClient;
import org.objectweb.petals.tools.jmx.api.PetalsJMXClient;
import org.objectweb.petals.tools.jmx.api.exception.ConnectionErrorException;
import org.objectweb.petals.tools.jmx.api.exception.DeploymentServiceDoesNotExistException;
import org.objectweb.petals.tools.jmx.api.exception.DeploymentServiceErrorException;
import org.objectweb.petals.tools.jmx.api.exception.PerformActionErrorException;
import org.objectweb.petals.tools.webadmin.bo.Message;
import org.objectweb.petals.tools.webadmin.bo.MessageExchange;
import org.objectweb.petals.tools.webadmin.bo.ServerConfiguration;
import org.objectweb.petals.tools.webadmin.util.JBIEntity;
import org.objectweb.petals.tools.webadmin.util.JBIJMXConnectorUtil;
import org.objectweb.petals.tools.webadmin.util.JMXUtils;
import org.objectweb.petals.tools.webadmin.util.MathTools;
import org.objectweb.petals.tools.webadmin.util.MessagesListener;
import org.objectweb.petals.tools.webadmin.util.ServerPropertiesManager;

/**
 * DataCollector is a singleton : only one can be instantiate and it's not
 * cloneable. Clients can only gets a reference to the existing DataCollector
 * and call methods through this reference.
 * 
 * @author msauvage
 * 
 */
public final class DataCollector {

    /**
     * label for List class name
     */
    private static final String CLASS_LIST = "java.util.List";

    /**
     * label for Map class name
     */
    private static final String CLASS_MAP = "java.util.Map";

    /**
     * label for ObjectName class name
     */
    private static final String CLASS_OBJECTNAME = "javax.management.ObjectName";

    /**
     * label for String class name
     */
    private static final String CLASS_STRING = "java.lang.String";

    /**
     * the single collector object
     */
    private static DataCollector collector;

    /**
     * label for "started"
     */
    private static final String STARTED = "started";

    // object for dates formating
    private static final DateFormat DATE_FORMATER = new SimpleDateFormat(
            "dd MMM yyyy, HH:mm:ss");

    /**
     * Name of the mbean storing servers on the storage jmx server
     */
    private ObjectName componentsMbeanName;

    /**
     * configuration of a known PEtALS server of the domain
     */
    private ServerConfiguration entryServerConf;

    /**
     * indicates if collector has been initialized or not
     */
    private boolean init;

    /**
     * Tool for JMX operations with servers
     */
    private JMXUtils jmxUtils;

    /**
     * last time of datas retrieval.
     */
    private Map<String, Long> lastRetrieveTime = new HashMap<String, Long>();

    /**
     * Name of the PEtALS administration MBean
     */
    private ObjectName petalsAdminName;

    /**
     * Name of the PEtALS monitoring MBean
     */
    private ObjectName petalsMonitoringName;

    /**
     * configuration of all started PEtALS servers on the domain
     */
    private List<ServerConfiguration> petalsServersConf;

    /**
     * manager of the properties file containing JMX servers informations
     */
    private ServerPropertiesManager propertiesManager;

    /**
     * Name of the mbean storing servers on the storage jmx server
     */
    private ObjectName serversListMbeanName;

    /**
     * Name of the storage administration MBean
     */
    private ObjectName storageAdminName;

    /**
     * configuration of datas storage JMX server
     */
    private ServerConfiguration storageServerConf;

    /**
     * stores the datas sets waiting to be exposed on storage server
     */
    private Map<String, Map<Long, Long>> timeDatasets = new HashMap<String, Map<Long, Long>>();

    /**
     * Name of the mbean storing time datas on the storage jmx server
     */
    private ObjectName timeDatasMBeanName;

    /**
     * Listeners to receive messages notifications from servers. Listeners are
     * stored as a Map<ServerName, listener>
     */
    private Map<String, MessagesListener> messagesListeners = new HashMap<String, MessagesListener>();
    
    /**
     * map of exchanges storages, key=serverName
     */
    private Map<String,ExchangesStorage> exchangesStorages= new HashMap<String, ExchangesStorage>();;

    /**
     * Creates a new empty DataCollector
     * 
     */
    private DataCollector() {
    }

    /**
     * Return the actual collector
     * 
     * @return
     */
    public static DataCollector getCollector() {
        if (collector == null) {
            collector = new DataCollector();
        }
        return collector;
    }

    /**
     * starts monitoring on a server of the list of servers via a JMX call on
     * the administration MBean of this server to call his activation method
     * 
     * @param serverName
     *            the name of the server to monitor
     * @param showMessageContent
     *            true to monitor also the content of messages, false to ignore
     *            it
     * @throws NoSuchFieldException
     *             thrown if server is not found in the list of servers
     * @throws MBeanException
     *             problem occured in communication with server when trying to
     *             sets the new monitoring attribute value
     */
    public void activateMonitoring(String serverName, boolean showMessageContent)
            throws NoSuchFieldException, MBeanException {
        ServerConfiguration serverConf = this.getServer(serverName);
        try {
            // activate monitoring in PEtALS
            serverConf.getConnection().invoke(petalsMonitoringName,
                    "activateMonitoring", new Object[] { showMessageContent },
                    new String[] { "boolean" });
        } catch (InstanceNotFoundException e) {
            e.printStackTrace(System.out);
            throw new MBeanException(e);
        } catch (ReflectionException e) {
            e.printStackTrace(System.out);
            throw new MBeanException(e);
        } catch (IOException e) {
            e.printStackTrace(System.out);
            throw new MBeanException(e);
        } catch (MBeanException e) {
            e.printStackTrace(System.out);
            throw new MBeanException(e);
        }
        // update server configuration
        serverConf.setMonitor(true);
        serverConf.setShowMessageContent(showMessageContent);
        // update server's configuration in storage server
        this.updateExposedServerConf(serverConf);
    }

    /**
     * starts monitoring on each server of the list of servers via a JMX call on
     * their administration MBean. If showMessageContent is set to true,
     * activate the monitoring of the content of messages.
     * 
     * @param showMessageContent
     *            true to monitor content of messages, false to ignore it
     * @throws NoSuchFieldException
     *             thrown if server is not found in the list of servers
     * @throws MBeanException
     */
    public void activateMonitoringAllServers(boolean showMessageContent)
            throws NoSuchFieldException, MBeanException {
        for (ServerConfiguration serverConf : petalsServersConf) {
            this.activateMonitoring(serverConf.getName(), showMessageContent);
        }
    }

    /**
     * Change the state of a component on a PEtALS server
     * 
     * @param serverName
     *            the name of the server on which the component is installed
     * @param compoName
     *            the name of the component
     * @param action
     *            the action to do : "start" or "stop"
     * @throws NoSuchFieldException
     * @throws MBeanException
     */
    public void changeCompoState(String serverName, String compoName,
            String action) throws NoSuchFieldException, MBeanException {
        try {
            ServerConfiguration serverConf = this.getServer(serverName);
            MBeanServerConnection connection = serverConf.getConnection();
            Object result = connection.invoke(JBIJMXConnectorUtil
                    .getAdminServiceMBeanName(connection),
                    "getComponentByName", new Object[] { compoName },
                    new String[] { CLASS_STRING });
            if (result instanceof ObjectName) {
                ObjectName compoObjName = (ObjectName) result;
                connection.invoke(compoObjName, action, new Object[] {},
                        new String[] {});
            }
        } catch (InstanceNotFoundException e) {
            throw new MBeanException(e);
        } catch (MalformedObjectNameException e) {
            throw new MBeanException(e);
        } catch (MBeanException e) {
            throw new MBeanException(e);
        } catch (ReflectionException e) {
            throw new MBeanException(e);
        } catch (IOException e) {
            throw new MBeanException(e);
        }
    }

    /**
     * starts monitoring on a server of the list of servers via a JMX call on
     * the administration MBean of this server to call his activation method
     * 
     * @param serverName
     *            the name of the server to monitor
     * @param showMessageContent
     *            true to monitor also the content of messages, false to ignore
     *            it
     * @throws NoSuchFieldException
     *             thrown if server is not found in the list of servers
     * @throws MBeanException
     *             problem occured in communication with server when trying to
     *             sets the new monitoring attribute value
     */
    public void deactivateMonitoring(String serverName)
            throws NoSuchFieldException, MBeanException {
        ServerConfiguration serverConf = this.getServer(serverName);
        try {
            // deactivate monitoring in PEtALS
            serverConf.getConnection().invoke(petalsMonitoringName,
                    "deactivateMonitoring", new Object[] {}, new String[] {});
        } catch (InstanceNotFoundException e) {
            throw new MBeanException(e);
        } catch (ReflectionException e) {
            throw new MBeanException(e);
        } catch (IOException e) {
            throw new MBeanException(e);
        }
        // update server's configuration
        serverConf.setMonitor(false);
        serverConf.setShowMessageContent(false);
        // update server's configuration on storage server
        this.updateExposedServerConf(serverConf);
    }

    /**
     * stops monitoring on each server of the list of servers via a JMX call on
     * their administration MBean.
     * 
     * @throws NoSuchFieldException
     *             thrown if server is not found in the list of servers
     * @throws MBeanException
     */
    public void deactivateMonitoringAllServers() throws NoSuchFieldException,
            MBeanException {
        for (ServerConfiguration serverConf : petalsServersConf) {
            this.deactivateMonitoring(serverConf.getName());
        }
    }

    /**
     * Retrieves via a JMX call to each server of the list of servers
     * petalsServersConf a <name,component> Map of his components installed and
     * expose them in the storage JMX server as a List in ServersComponents
     * MBean
     * 
     * @throws IOException
     *             storage server is not reachable
     * @throws MBeanException
     *             invocation of MBean in storage server threw an exception
     */
    public void updateAndExposeComponents() throws IOException, MBeanException {
        if (!this.storageServerReachable()) {
            throw new IOException();
        }
        try {
            Map<String, Map<String, Map<String, Object>>> serversComponents = new HashMap<String, Map<String, Map<String, Object>>>();
            // population global set of <server,components>
            for (ServerConfiguration serverConf : petalsServersConf) {
                if (serverConf.getState().equalsIgnoreCase((STARTED))) {
                    // getting component from current server
                    Map<String, JBIEntity> components = jmxUtils
                            .listComponents(serverConf.getConnection());
                    // Map containing components as <name,component>
                    Map<String, Map<String, Object>> componentList = new HashMap<String, Map<String, Object>>();
                    // populating map
                    for (String componentName : components.keySet()) {
                        // adding component as a Map to the map of components
                        componentList.put(componentName, components.get(
                                componentName).toMap());
                    }
                    // adding components to the global set
                    serversComponents.put(serverConf.getName(), componentList);
                }
            }

            // creating components MBean via admin MBean
            storageServerConf.getConnection().invoke(storageAdminName,
                    "initComponentsMBean",
                    new Object[] { componentsMbeanName, serversComponents },
                    new String[] { CLASS_OBJECTNAME, CLASS_MAP });
        } catch (InstanceNotFoundException infe) {
            throw new MBeanException(infe);
        } catch (MBeanException mbe) {
            throw new MBeanException(mbe);
        } catch (ReflectionException re) {
            throw new MBeanException(re);
        } catch (IOException ioe) {
            throw new MBeanException(ioe);
        }
    }

    /**
     * Gets the configuration of PEtALS entry server
     * 
     * @return a ServerConfiguration containing server's informations
     */
    public ServerConfiguration getEntryServerConf() {
        return entryServerConf;
    }

    /**
     * Gets admin MBean name of PEtALS
     * 
     * @return an Objectname representation of the name of the MBean
     */
    public ObjectName getPetalsAdminName() {
        return petalsAdminName;
    }

    /**
     * Gets configuration of all servers
     * 
     * @return a List of ServerConfiguration containing informations about all
     *         servers of the domain
     */
    public List<ServerConfiguration> getPetalsServersConf() {
        return petalsServersConf;
    }

    /**
     * Gets the manager of properties file
     * 
     * @return the properties manager
     */
    public ServerPropertiesManager getPropertiesManager() {
        return propertiesManager;
    }

    /**
     * Gets a server configuration from the list of servers by his name
     * 
     * @param serverName
     *            the name of the server wanted
     * @return a ServerConfiguration of the server
     * @throws NoSuchFieldException
     *             server doesn't exist
     * @throws NullPointerException
     *             serverName parameter is null
     */
    public ServerConfiguration getServer(String serverName)
            throws NoSuchFieldException {
        if (serverName == null) {
            throw new NullPointerException("serverName is null");
        }
        for (ServerConfiguration serverConf : petalsServersConf) {
            if (serverConf.getName().equalsIgnoreCase(serverName)) {
                return serverConf;
            }
        }
        throw new NoSuchFieldException("Server " + serverName + " not found");
    }

    /**
     * Gets the configuration of storage server
     * 
     * @return a ServerConfiguration containing server's information
     */
    public ServerConfiguration getStorageServerConf() {
        return storageServerConf;
    }

    /**
     * Initialize the DataCollector.
     * 
     * Sets a new ServerPropertiesManager and a initialize petals entry server
     * and datas storage JMX server configurations by reading their properties
     * via the ServerPropertiesManager. Initialize the list of servers and
     * exposes on the storage server their properties and components
     * 
     * @throws IOException
     *             the properties manager file is not readable or is not found
     * @throws MBeanException
     *             problem occured in communication with storage server
     * @throws AttributeNotFoundException 
     */
    public void init(String rootPath) throws MBeanException, IOException, AttributeNotFoundException {
        this.log("INFO", "Initializing DataCollector with rootPath :"
                + rootPath);
        propertiesManager = new ServerPropertiesManager(rootPath);
        jmxUtils = JMXUtils.getJMXUtils();

        boolean loaded = true;
        loaded = propertiesManager.load();
        if (loaded) {
            // initiliazing servers informations
            this.initEntryServer();
            this.initStorageServer();

            // initializing mbeans
            this.initMBeanNames();
            this.initMBeans();

            if (entryServerConf.getConnection() != null) {
                // retrieve and expose list of servers on storage jmx server
                this.updateServers();
                // expose components of servers on storage jmx server
                this.updateAndExposeComponents();
                // create listeners on servers
                this.initListeners();
            }
        }
        init = true;
    }

    /**
     * Install a component on a given server. Component is on its archive (.zip)
     * form and is located by its path on the machine where is hosted the
     * webadmin application
     * 
     * @param serverName
     *            the name of the server where to install component
     * @param filePath
     *            complete path of the archive on the server hosting the webapp
     * @throws MBeanException
     *             a problem occured with server during the installation
     */
    public void installComponent(String serverName, String filePath)
            throws MBeanException {
        try {
            String url = filePath;
            ServerConfiguration serverConf = this.getServer(serverName);
            MBeanServerConnection connection = serverConf.getConnection();
            Object result = connection.invoke(JBIJMXConnectorUtil
                    .getInstallationServiceMBeanName(connection),
                    "loadNewInstaller", new Object[] { url },
                    new String[] { CLASS_STRING });
            if (result instanceof ObjectName) {
                ObjectName installerName = (ObjectName) result;
                // Install the component
                connection.invoke(installerName, "install", new Object[] {},
                        new String[] {});
            } else {
                throw new MBeanException(new Exception(
                        "file is not a component"));
            }
        } catch (IOException e) {
            throw new MBeanException(e);
        } catch (InstanceNotFoundException e) {
            throw new MBeanException(e);
        } catch (MalformedObjectNameException e) {
            throw new MBeanException(e);
        } catch (ReflectionException e) {
            throw new MBeanException(e);
        } catch (NoSuchFieldException e) {
            throw new MBeanException(e);
        }
    }

    /**
     * Temporary method to install a new Service Assembly
     * 
     */
    public void installServiceAssembly(String serverName, String filepath)
            throws MBeanException {
        PetalsJMXClient petalsJMXClient;
        try {
            ServerConfiguration serverConf = this.getServer(serverName);
            String host = serverConf.getHost();
            int port = Integer.parseInt(serverConf.getJmxPort());
                      
            petalsJMXClient = new PetalsJMXClient(host, port, "", "");

            PetalsDeploymentServiceClient pdsc = petalsJMXClient
                    .getDeploymentServiceClient();

            /*******************************************************************
             * boolean serviceAssemblyFound = pdsc
             * .isServiceAssemblyDeployed(serviceAssemblyName); if
             * (serviceAssemblyFound) { throw new MBeanException( null, "The
             * service-assembly is already deployed"); }
             ******************************************************************/
            
            URL jbiPackageURL = new URL(filepath);
            pdsc.deploy(jbiPackageURL);

        } catch (ConnectionErrorException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (DeploymentServiceDoesNotExistException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (DeploymentServiceErrorException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (PerformActionErrorException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (MalformedURLException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        }
    }
    
    /**
     * Temporary method to uninstall a new Service Assembly
     * 
     */
    public void uninstallServiceAssembly(String serverName, String saName)
            throws MBeanException {
        PetalsJMXClient petalsJMXClient;
        try {
            ServerConfiguration serverConf = this.getServer(serverName);
            String host = serverConf.getHost();
            int port = Integer.parseInt(serverConf.getJmxPort());
                      
            petalsJMXClient = new PetalsJMXClient(host, port, "", "");

            PetalsDeploymentServiceClient pdsc = petalsJMXClient
                    .getDeploymentServiceClient();

            pdsc.undeploy(saName);

        } catch (ConnectionErrorException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (DeploymentServiceDoesNotExistException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (DeploymentServiceErrorException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (PerformActionErrorException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        }
    }

    /**
     * Temporary method to change the state of a new Service Assembly
     * 
     */
    public void changeSAState(String serverName, String saName,String operation)
            throws MBeanException {
        PetalsJMXClient petalsJMXClient;
        try {
            ServerConfiguration serverConf = this.getServer(serverName);
            String host = serverConf.getHost();
            int port = Integer.parseInt(serverConf.getJmxPort());
                      
            petalsJMXClient = new PetalsJMXClient(host, port, "", "");

            PetalsDeploymentServiceClient pdsc = petalsJMXClient
                    .getDeploymentServiceClient();
            
            if (("stop").equalsIgnoreCase(operation)) {
                pdsc.stop(saName);
            } else if (("start").equalsIgnoreCase(operation)) {
                pdsc.start(saName);
            } else if (("shutdown").equalsIgnoreCase(operation)) {
                pdsc.shutdown(saName);
            } 

        } catch (ConnectionErrorException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (DeploymentServiceDoesNotExistException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (DeploymentServiceErrorException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (PerformActionErrorException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        }
    }

    /**
     * Indicates if DataCollector have successfully been initialized by init()
     * method
     * 
     * @return true if init() method have been called and have successfully
     *         initialized the collector, false if not
     */
    public boolean isInit() {
        return init;
    }

    public boolean isMessageContentShown(String serverName)
            throws MBeanException, NoSuchFieldException {
        ServerConfiguration serverConf = this.getServer(serverName);
        boolean isShown = false;
        try {
            isShown = (Boolean) serverConf.getConnection().getAttribute(
                    petalsMonitoringName, "MessageContentShown");
        } catch (InstanceNotFoundException e) {
            throw new MBeanException(e);
        } catch (ReflectionException e) {
            throw new MBeanException(e);
        } catch (IOException e) {
            throw new MBeanException(e);
        } catch (AttributeNotFoundException e) {
            throw new MBeanException(e);
        }
        return isShown;
    }

    /**
     * Indicates if monitoring is active or not on a server by retrieving the
     * information on the server himself.
     * 
     * @return true if monitoring is running, false if not.
     */
    public boolean isMonitoring(String serverName) throws MBeanException,
            NoSuchFieldException {
        ServerConfiguration serverConf = this.getServer(serverName);
        boolean isMonitoring = false;
        try {
            isMonitoring = (Boolean) serverConf.getConnection().getAttribute(
                    petalsMonitoringName, "Monitoring");
        } catch (InstanceNotFoundException e) {
            throw new MBeanException(e);
        } catch (ReflectionException e) {
            throw new MBeanException(e);
        } catch (IOException e) {
            throw new MBeanException(e);
        } catch (AttributeNotFoundException e) {
            throw new MBeanException(e);
        }
        return isMonitoring;
    }

    /**
     * Process a new set of messages.
     * 
     * @param messagesList
     *            a list of Message object representations of messages. Must not
     *            be null.
     * @throws IOException
     * @throws MBeanException
     */
    public void processNewDatas(List<Message> messagesList) {
        if (messagesList.size() > 0) {
            // extract server name from first message
            String serverName = messagesList.get(0).getContainer();
            // process messages by second and add to datasets
            timeDatasets.put("MsgsPerSecond-Server" + serverName, this
                    .getMsgsBySecond(messagesList));
             try {
                this.exposeTimeDatas();
                this.clearTimeDatas();
            } catch (MBeanException e) {
                System.out.println("Error processing datas :");
                e.printStackTrace(System.out);
            } catch (IOException e) {
                System.out.println("Error processing datas :");
                e.printStackTrace(System.out);
            }
        }
    }

    /**
     * Sets the PEtALS entry server configuration.
     * 
     * Sets it as properties of the DataCollector and register the informations
     * in a properties file
     * 
     * @param serverConf
     *            a server's configuration
     * @throws IOException
     */
    public void setEntryServerConf(ServerConfiguration serverConf)
            throws IOException {
        entryServerConf = serverConf;
        propertiesManager.setAttributes(serverConf.getConnectionParameters());
        propertiesManager.save();
    }

    /**
     * Sets the last time of retrieval for a server by associating the name of
     * the server as key to a timestamp as the time. Both key and values must be
     * not null.
     * 
     * @param serverName
     *            the name of the server
     * @param timestamp
     *            the time to associate with the server's name
     */
    public void setLastRetrieveTime(String serverName, long timestamp) {
        lastRetrieveTime.put(serverName, timestamp);
    }

    /**
     * Sets the admin MBean name of PEtALS
     * 
     * @param petalsAdminName
     *            an Objectname representation of the name of the MBean
     */
    public void setPetalsAdminName(ObjectName petalsAdminName) {
        this.petalsAdminName = petalsAdminName;
    }

    /**
     * Sets the list of PEtALS servers to a new list.
     * 
     * @param petalsServersConf
     *            a list of ServerConfiguration representation of the
     *            informations of each server
     */
    public void setPetalsServersConf(List<ServerConfiguration> petalsServersConf) {
        this.petalsServersConf = petalsServersConf;
    }

    /**
     * Sets the properties manager to a new ServerPropertiesManager
     * 
     * @param propertiesManager
     *            a ServerPropertiesManager representation of a properties
     *            manager
     */
    public void setPropertiesManager(ServerPropertiesManager propertiesManager) {
        this.propertiesManager = propertiesManager;
    }

    /**
     * Sets the configuration of datas storage server
     * 
     * @param serverConf
     *            a ServerConfiguration containing informations of the JMX
     *            Server to set
     */
    public void setStorageServerConf(ServerConfiguration serverConf) {
        this.storageServerConf = serverConf;
    }

    /**
     * Uninstall a component on a PEtALS server
     * 
     * @param serverName
     *            the name of the PEtALS server on which component is installed
     * @param compoName
     *            the name of the component to uninstall
     * @throws MBeanException
     */
    public void uninstallComponent(String serverName, String compoName)
            throws MBeanException {
        try {
            ServerConfiguration serverConf = this.getServer(serverName);
            MBeanServerConnection connection = serverConf.getConnection();
            connection.invoke(JBIJMXConnectorUtil
                    .getInstallationServiceMBeanName(connection),
                    "forceUnloadComponentInstaller",
                    new Object[] { compoName }, new String[] { CLASS_STRING });
        } catch (IOException e) {
            throw new MBeanException(e);
        } catch (InstanceNotFoundException e) {
            throw new MBeanException(e);
        } catch (MalformedObjectNameException e) {
            throw new MBeanException(e);
        } catch (ReflectionException e) {
            throw new MBeanException(e);
        } catch (NoSuchFieldException e) {
            throw new MBeanException(e);
        }
    }

    /**
     * Update a server's configuration in the ServersList mbean on the storage
     * server
     * 
     * @param serverConf
     *            the new server's configuration
     * @throws MBeanException
     */
    public void updateExposedServerConf(ServerConfiguration serverConf)
            throws MBeanException {
        try {
            storageServerConf.getConnection().invoke(serversListMbeanName,
                    "updateServer",
                    new Object[] { serverConf.getServerProperties() },
                    new String[] { CLASS_MAP });
        } catch (InstanceNotFoundException e) {
            throw new MBeanException(e);
        } catch (MBeanException e) {
            throw new MBeanException(e);
        } catch (ReflectionException e) {
            throw new MBeanException(e);
        } catch (IOException e) {
            throw new MBeanException(e);
        }
    }

    /**
     * Update the list of servers petalsServersConf and expose the list on the
     * storage JMX server by retrieving the informations of all servers via a
     * JMX call to the entry server.
     * 
     * Update their monitor status after retrieval. If an exception occures
     * while retrieving the monitor status, it stays to null and servers are
     * exposed anyway.
     * 
     * @throws IOException
     *             one of the entry server or the storage server is not
     *             reachable
     * @throws MBeanException
     *             re-throwing exception received from exposing servers
     * @throws AttributeNotFoundException 
     */
    public void updateServers() throws IOException, MBeanException {
        if (this.entryServerReachable()) {
            try {
                petalsServersConf = jmxUtils.getAllServers(entryServerConf);
            } catch (AttributeNotFoundException e1) {
                // TODO Auto-generated catch block
                e1.printStackTrace();
            }
            // if there are servers, searching for their monitoring status
            if (petalsServersConf != null && petalsServersConf.size() > 0) {
                for (ServerConfiguration serverConf : petalsServersConf) {
                    if (serverConf.getState().equalsIgnoreCase(PetalsAdminServiceMBean.STATE_STARTED)) {
                        try {
                            this.updateMonitorStatus(serverConf);
                        } catch (AttributeNotFoundException e) {
                            e.printStackTrace(System.err);
                        } catch (MBeanException e) {
                            e.printStackTrace(System.err);
                        } // nothing to do if update of monitor status fails
                    }
                }
            }
            this.updateListeners();
            this.exposeServers();
            this.updateAndExposeComponents();
        } else {
            throw new IOException("PEtALS entry server unreachable : "
                    + this.entryServerConf.toString());
        }
    }

    /**
     * Updates the listeners stored : removes listeners to server unexisting or
     * stopped and adds listeners to new active servers
     * 
     * @throws MBeanException
     *             an exception occured when trying to create a listener
     * @see #createListener(ServerConfiguration)
     * @see #messagesListeners
     */
    public void updateListeners() throws MBeanException {
        // list to store names of server to remove to prevent concurrent
        // modification with removing while iterating
        List<String> toRemove = new ArrayList<String>();
        // comparing servers list to listener list
        for (ServerConfiguration serverConf : petalsServersConf) {
            String serverName = serverConf.getName();
            if (serverConf.getState().equalsIgnoreCase(PetalsAdminServiceMBean.STATE_STARTED)
                    && messagesListeners.get(serverName) == null) {
                createListener(serverConf);
            } else if (serverConf.getState().equalsIgnoreCase(PetalsAdminServiceMBean.STATE_STOPPED)
                    && messagesListeners.get(serverName) != null) {
                toRemove.add(serverName);
            }
        }
        // comparing listeners to servers list
        for (String serverName : messagesListeners.keySet()) {
            boolean serverFound = false;
            for (ServerConfiguration serverConf : petalsServersConf) {
                // searching for server
                if (serverConf.getName().equalsIgnoreCase(serverName)) {
                    serverFound = true;
                }
            }
            if (!serverFound) {
                toRemove.add(serverName);
            }
        }
        // removing listeners marked for removal
        for (String serverName : toRemove) {
            messagesListeners.remove(serverName);
        }
    }

    /**
     * Creates a listener to a server and stores it in the messagesListeners map
     * 
     * @param serverConf
     *            configuration of the server to listen
     * @throws MBeanException
     *             a problem occurs while communicating with the server
     * @see #messagesListeners
     */
    protected void createListener(ServerConfiguration serverConf)
            throws MBeanException {
        MessagesListener listener = new MessagesListener(this);
        try {
            serverConf.getConnection().addNotificationListener(
                    new ObjectName("Petals:name=Monitoring,type=service"),
                    listener, null, null);
        } catch (InstanceNotFoundException e) {
            throw new MBeanException(e);
        } catch (MalformedObjectNameException e) {
            throw new MBeanException(e);
        } catch (IOException e) {
            throw new MBeanException(e);
        }
        messagesListeners.put(serverConf.getName(), listener);
    }

    /**
     * Clears all time datas sets registered
     * 
     */
    private void clearTimeDatas() {
        timeDatasets.clear();
    }

    /**
     * Verifies if the entry server is reachable.
     * 
     * i.e. if it has a connection and is accessible via this connection. Try to
     * create one if it hasn't, resets connection to null if it isn't
     * accessible. Return false if none of the operations succeeded.
     * 
     * @return true if server has a connection and is accessible, false if not.
     */
    private boolean entryServerReachable() {
        boolean serverReachable = true;
        if (entryServerConf.getConnection() == null) {
            try {
                entryServerConf.createConnection();
                serverReachable = true;
            } catch (IOException ioe) {
                serverReachable = false;
            }
        }
        if (!jmxUtils.petalsServerReachable(entryServerConf)) {
            entryServerConf.setConnection(null);
            serverReachable = false;
        }
        return serverReachable;
    }

    /**
     * Expose the actual list of petals servers in the storage JMX server in the
     * ServersConfigurationList MBean as a list of Map representation of their
     * informations. Do nothing if list is null or empty.
     * 
     * @throws IOException
     *             storage server is not reachable
     * @throws MBeanException
     *             invocation of MBean in storage server threw an exception
     */
    private void exposeServers() throws MBeanException, IOException {
        if (!this.storageServerReachable()) {
            throw new IOException();
        }
        if (petalsServersConf != null && petalsServersConf.size() > 0) {
            // transforming list of ServerConfiguration into a
            // list of HashMap for serialization
            List<Map<String, String>> listServersConf = new ArrayList<Map<String, String>>();
            for (ServerConfiguration currentPetals : petalsServersConf) {
                listServersConf.add(currentPetals.getServerProperties());
            }
            // creating servers MBean via admin MBean
            try {
                storageServerConf.getConnection().invoke(storageAdminName,
                        "initServersMBean",
                        new Object[] { serversListMbeanName, listServersConf },
                        new String[] { CLASS_OBJECTNAME, CLASS_LIST });
            } catch (InstanceNotFoundException infe) {
                throw new MBeanException(infe);
            } catch (MBeanException mbe) {
                throw new MBeanException(mbe);
            } catch (ReflectionException re) {
                throw new MBeanException(re);
            } catch (IOException ioe) {
                throw new MBeanException(ioe);
            }
        }
    }

    /**
     * Exposes the time datas registered in timeDatasets in an MBean on the
     * storage JMX server. Creates the MBean if it doesn't already exists.
     * 
     * @throws IOException
     *             the storage server is unreachable
     * @throws MBeanException
     *             problem occured when communicating with server
     */
    private void exposeTimeDatas() throws IOException, MBeanException {
        if (!this.storageServerReachable()) {
            throw new IOException();
        }
        try {
            // if MBean already exists -> adding all datas for each server
            // else -> creating MBean with all datas
            if (storageServerConf.getConnection().isRegistered(
                    timeDatasMBeanName)) {
                for (String key : timeDatasets.keySet()) {
                    // store new datas in MBean
                    storageServerConf.getConnection().invoke(
                            timeDatasMBeanName, "addAll",
                            new Object[] { key, timeDatasets.get(key) },
                            new String[] { CLASS_STRING, CLASS_MAP });
                }
            } else {
                // creating the MBean via admin MBean
                storageServerConf.getConnection().invoke(storageAdminName,
                        "initTimeDatasMBean",
                        new Object[] { timeDatasMBeanName, timeDatasets },
                        new String[] { CLASS_OBJECTNAME, CLASS_MAP });
            }
        } catch (InstanceNotFoundException e) {
            throw new MBeanException(e);
        } catch (MBeanException e) {
            throw new MBeanException(e);
        } catch (ReflectionException e) {
            throw new MBeanException(e);
        }
    }

    /**
     * Gets the number of messages per second for a list of exchanges
     * 
     * @param exchList
     *            the list of MessageExchanges to count
     * @return a Map associating timestamp of each second with a number of
     *         messages
     */
    private Map<Long, Long> getMsgsBySecond(List<Message> messagesList) {
        Map<Long, Long> mbs = new HashMap<Long, Long>();
        // control list to count messages only one time
        Map<String, String> messagesCounted = new HashMap<String, String>();
        // iterating messages via exchanges
        for (Message message : messagesList) {

            String type = message.getType();
            // transforming null type to a String
            if (message.getType() == null) {
                type = "null";
            }

            // creating the control key
            String key = message.getId() + "_" + type;
            // searching in the selection list a message with this key
            String searchedType = messagesCounted.get(key);

            // if there is no message with this key or if the message found
            // has a different type -> couting message to the list and
            // adding it in messages counted
            if (searchedType == null || !searchedType.equalsIgnoreCase(type)) {
                // rounding timestamp to second
                long second = (message.getTime() / MathTools.THOUSAND)
                        * MathTools.THOUSAND;
                // if second already exists -> add 1 to count
                // else -> create an entry with 1 in count
                if (mbs.get(second) != null) {
                    mbs.put(second, mbs.get(second) + 1);
                } else {
                    mbs.put(second, 1L);
                }
                messagesCounted.put(key, type);
            }
        }
        return mbs;
    }

    public Map<String,MessageExchange> getMessagesExchanges(String serverName) {
        Map<String,MessageExchange> ex = null; 
        if (exchangesStorages.containsKey(serverName)) {
            ex=exchangesStorages.get(serverName).getExchanges();
        }
        return ex;
    }
    
   public MessageExchange getMessageExchange(String serverName,String exchangeId) {
        MessageExchange ex = null; 
        if (exchangesStorages.containsKey(serverName)) {
            Map<String, MessageExchange> maps = exchangesStorages.get(serverName).getExchanges();
            if (maps.containsKey(exchangeId)) {
                ex = maps.get(exchangeId);
            }
        }
        return ex;
    }
    
    /**
     * Initialize PEtALS entry server for the domain by reading properties in
     * file via the properties manager. Create a connection for it if server is
     * reachable, if server is not reachable it's connection attribute stays
     * null.
     * 
     */
    private void initEntryServer() {
        entryServerConf = new ServerConfiguration();
        entryServerConf.setHost(propertiesManager
                .getAttributeValue("serverHost"));
        entryServerConf.setJmxLogin(propertiesManager
                .getAttributeValue("jmxLogin"));
        entryServerConf.setJmxPort(propertiesManager
                .getAttributeValue("jmxPort"));
        entryServerConf.setJmxPassword(propertiesManager
                .getAttributeValue("jmxPassword"));
        try {
            entryServerConf.createConnection();
        } catch (IOException e) {
            System.out
                    .println("[WEBADMIN WARN] PEtALS entry server unreachable : "
                            + entryServerConf.toString());
        }
    }

    /**
     * initializes the listeners to every Petals server started known to receive
     * notifications emitted by their notification broadcast support
     * 
     * @throws MBeanException
     */
    private void initListeners() throws MBeanException {
        for (ServerConfiguration serverConf : petalsServersConf) {
            if (serverConf.getState().equalsIgnoreCase(PetalsAdminServiceMBean.STATE_STARTED)) {
                createListener(serverConf);
            }
        }
    }

    /**
     * Creates "base" mbeans needed on storage server
     * 
     * @throws IOException
     * @throws MBeanException
     * 
     */
    private void initMBeans() throws IOException, MBeanException {
        if (this.storageServerReachable()) {
            try {
                if (!storageServerConf.getConnection().isRegistered(
                        timeDatasMBeanName)) {
                    // creating the MBean via admin MBean
                    storageServerConf.getConnection().invoke(storageAdminName,
                            "initTimeDatasMBean",
                            new Object[] { timeDatasMBeanName, timeDatasets },
                            new String[] { CLASS_OBJECTNAME, CLASS_MAP });
                }
            } catch (InstanceNotFoundException e) {
                throw new MBeanException(e);
            } catch (MBeanException e) {
                throw new MBeanException(e);
            } catch (ReflectionException e) {
                throw new MBeanException(e);
            }
        }
    }

    /**
     * Sets the administration MBean name of the storage server.
     * 
     * If storageServerConf or its connection is null, the admin MBean name
     * stays null
     * 
     * @param mBeanName
     *            the name of the MBean - must be not null
     * @throws IOException
     * @throws MalformedObjectNameException
     *             The string used to create the ObjectName does not have the
     *             right format.
     * @throws IOException
     *             a communication error occured when retrieving de
     *             DefaultDomain from the storage server
     */
    private void initMBeanNames() throws IOException {
        if (this.storageServerReachable()) {
            try {
                // petals mbeans
                petalsAdminName = new ObjectName(
                        "Petals:name=PetalsAdmin,type=service");
                petalsMonitoringName = new ObjectName(
                        "Petals:name=Monitoring,type=service");

                // storage admin
                storageAdminName = new ObjectName(storageServerConf
                        .getConnection().getDefaultDomain()
                        + ":name=StorageAdmin,type=service");

                // init name of the Mbean that stores components on storage
                // server
                componentsMbeanName = new ObjectName(storageServerConf
                        .getConnection().getDefaultDomain()
                        + ":name=ServersComponents,type=ComponentsList");
                // init name of the Mbean that stores the servers conf
                serversListMbeanName = new ObjectName(storageServerConf
                        .getConnection().getDefaultDomain()
                        + ":name=ServersConfigurationList,type=ServersList");
                // init name of the datas MBean to store
                timeDatasMBeanName = new ObjectName(storageServerConf
                        .getConnection().getDefaultDomain()
                        + ":name=MsgsByTime,type=TimeDatas");
            } catch (MalformedObjectNameException e) {
                e.printStackTrace(System.err);
            }
        }
    }

    /**
     * Initialize datas storage server for the domain by reading properties in
     * file via the properties manager. Create a connection for it if server is
     * reachable, if server is not reachable it's connection attribute stays
     * null.
     * 
     */
    private void initStorageServer() {
        storageServerConf = new ServerConfiguration();
        storageServerConf.setHost(propertiesManager
                .getAttributeValue("storageServerHost"));
        storageServerConf.setJmxLogin(propertiesManager
                .getAttributeValue("storageJmxLogin"));
        storageServerConf.setJmxPort(propertiesManager
                .getAttributeValue("storageJmxPort"));
        storageServerConf.setJmxPassword(propertiesManager
                .getAttributeValue("storageJmxPassword"));

        try {
            storageServerConf.createConnection();
        } catch (IOException e) {
            e.printStackTrace(System.err);
        }
    }

    /**
     * Verifies if storage server is reachable.
     * 
     * i.e. if it has a connection and is accessible via this connection. Try to
     * create one if it hasn't, resets connection to null if it isn't
     * accessible. Return false if none of the operations succeeded.
     * 
     * @return true if server has a connection and is accessible, false if not.
     */
    private boolean storageServerReachable() {
        boolean serverReachable = true;
        if (storageServerConf.getConnection() == null) {
            try {
                storageServerConf.createConnection();
                serverReachable = true;
            } catch (IOException ioe) {
                serverReachable = false;
            }
        }
        if (!jmxUtils.storageServerReachable(storageServerConf)) {
            storageServerConf.setConnection(null);
            serverReachable = false;
        }
        return serverReachable;
    }

    /**
     * update the monitor status of a server by retrieving it on the server and
     * setting his attribute in the list of servers
     * 
     * @param serverConf
     *            the server to monitor
     * @throws AttributeNotFoundException
     *             thrown if monitoring attribute is not found on the server
     * @throws MBeanException
     *             a problem occured in communication with server
     */
    private void updateMonitorStatus(ServerConfiguration serverConf)
            throws AttributeNotFoundException, MBeanException {
        boolean monitor = false;
        boolean showMessageContent = false;
        String serverName = serverConf.getName();
        if (serverConf.getState().equalsIgnoreCase(STARTED)) {
            try {
                monitor = this.isMonitoring(serverName);
                showMessageContent = this.isMessageContentShown(serverName);
            } catch (NoSuchFieldException e) {
                e.printStackTrace(System.err);
            }
        }
        serverConf.setMonitor(monitor);
        serverConf.setShowMessageContent(showMessageContent);
    }

    /**
     * debug function to see the content of the messages in a list of messages
     * 
     * @param messages
     *            the list of messages
     */
    @SuppressWarnings("unused")
    public void showMessagesInConsole(List<Map<String, Object>> messages) {
        DateFormat df = new SimpleDateFormat("EEE. dd MMM. yyyy, HH:mm:ss.SSS",
                Locale.ENGLISH);
        System.out.println(messages.size() + " messages in the list");
        for (Map<String, Object> mapMsg : messages) {
            Message message = new Message(mapMsg);
            if (message.getContent() == null) {
                System.out.println("  . " + df.format(message.getTime())
                        + " : " + message.getContainer() + " : "
                        + message.getEndpointName() + " : " + message.getType()
                        + " : -no content-");
            } else {
                System.out.println("  . " + df.format(message.getTime())
                        + " : " + message.getContainer() + " : "
                        + message.getEndpointName() + " : " + message.getType()
                        + " : " + message.getContent());
            }
        }
    }

    private void log(String level, String message) {
     /**   System.out.println("[WEBADMIN " + level + "] "
                + DATE_FORMATER.format(new Date().getTime()) + " : " + message);**/
    }

    public void storeMessages(List<Message> list,String notificationNumber) {
        if (exchangesStorages == null) {
            exchangesStorages = new HashMap<String, ExchangesStorage>();
        }
        if ((list != null) && (!(list.isEmpty()))) {
            String server = list.get(0).getContainer();
            ExchangesStorage storage;
        if (exchangesStorages.containsKey(server)) {
            storage=exchangesStorages.get(server);
        } else {
            storage = new ExchangesStorage(server);
            exchangesStorages.put(server, storage);
        }
        if (!(storage.hasProcessedNotification(notificationNumber))) {
            storage.addMessages(list);
            storage.addNotificationProcessed(notificationNumber);
        }
        }
    }

}
