/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: NewJMXService.java msauvage $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.webadmin.service;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServer;
import javax.management.MBeanServerConnection;
import javax.management.MBeanServerFactory;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.objectweb.petals.kernel.admin.PetalsAdminServiceMBean;
import org.objectweb.petals.tools.jmx.api.PetalsDeploymentServiceClient;
import org.objectweb.petals.tools.jmx.api.PetalsJMXClient;
import org.objectweb.petals.tools.jmx.api.exception.ConnectionErrorException;
import org.objectweb.petals.tools.jmx.api.exception.DeploymentServiceDoesNotExistException;
import org.objectweb.petals.tools.jmx.api.exception.DeploymentServiceErrorException;
import org.objectweb.petals.tools.jmx.api.exception.PerformActionErrorException;
import org.objectweb.petals.tools.webadmin.bo.ServerConfiguration;
import org.objectweb.petals.tools.webadmin.util.JBIEntity;
import org.objectweb.petals.tools.webadmin.util.JMXUtils;

public final class NewJMXService {

    /**
     * label for String class name
     */
    private static final String CLASS_STRING = "java.lang.String";

    /**
     * the singleton object
     */
    private static NewJMXService jmxService;

    private static final String SERVERS_CONF_MBEAN_NAME = ":name=ServersConfigurationList,type=ServersList";

    private JMXUtils jmxUtils;

    /**
     * the configuration of the PEtALS server to interact with
     */
    private ServerConfiguration serverConf;

    private NewJMXService(ServerConfiguration newServerConf) {
        serverConf = newServerConf;
        jmxUtils = JMXUtils.getJMXUtils();
    }

    /**
     * Creates a new JMXService connected to a new server
     * 
     * @param serverConf
     *            the server to contact
     */
    public static void createNewJmxService(ServerConfiguration serverConf) {
        jmxService = new NewJMXService(serverConf);
    }

    /**
     * Gets the singleton NewJMXService
     * 
     * @return a NewJMXService
     */
    public static NewJMXService getJmxService() {
        return jmxService;
    }

    /**
     * Gets the components installed on a server
     * 
     * @param serverName
     *            the server's name
     * @return a Map representation of the components as <name,component> items
     * @throws IOException
     * @throws MBeanException
     */
    @SuppressWarnings("unchecked")
    public Map<String, JBIEntity> getComponents(String serverName)
        throws IOException, MBeanException {
        // if ServerConfiguration has no connection, try to opens a new one
        // if unable to open one -> throws the IOException received
        if (!jmxUtils.storageServerReachable(serverConf)) {
            throw new IOException("Server " + serverConf.getName()
                + " not reachable");
        }
        MBeanServerConnection connection = serverConf.getConnection();
        try {
            ObjectName compoMBeanName = new ObjectName(connection
                .getDefaultDomain()
                + ":name=ServersComponents,type=ComponentsList");
            Map<String, Map<String, Object>> components = (Map<String, Map<String, Object>>) connection
                .invoke(compoMBeanName, "getComponents",
                    new Object[] {serverName}, new String[] {CLASS_STRING});
            Map<String, JBIEntity> serverComponents = new HashMap<String, JBIEntity>();
            if (components != null) {
                for (String currentCompoName : components.keySet()) {
                    serverComponents.put(currentCompoName, new JBIEntity(components
                        .get(currentCompoName)));
                }
            }
            return serverComponents;
        } catch (MalformedObjectNameException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (NullPointerException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (InstanceNotFoundException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (MBeanException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (ReflectionException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        }
    }

    /**
     * Gets the components installed on a server
     * 
     * @param serverName
     *            the server's name
     * @return a Map representation of the components as <name,component> items
     * @throws IOException
     * @throws MBeanException
     */
    @SuppressWarnings("unchecked")
    public Map<String, Map<String, Object>> getRawComponents(String serverName)
        throws MBeanException, IOException {
        // if ServerConfiguration has no connection, try to opens a new one
        // if unable to open one -> throws the IOException received
        if (!jmxUtils.storageServerReachable(serverConf)) {
            throw new IOException("Server " + serverConf.getName()
                + " not reachable");
        }
        MBeanServerConnection connection = serverConf.getConnection();
        try {
            ObjectName compoMBeanName = new ObjectName(connection
                .getDefaultDomain()
                + ":name=ServersComponents,type=ComponentsList");
            Map<String, Map<String, Object>> components = (Map<String, Map<String, Object>>) connection
                .invoke(compoMBeanName, "getComponents",
                    new Object[] {serverName}, new String[] {CLASS_STRING});
            return components;
        } catch (MalformedObjectNameException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (NullPointerException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (InstanceNotFoundException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (MBeanException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (ReflectionException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        }
    }

    /**
     * Gets the server configuration of a server by his name
     * 
     * @param serverName
     *            the name of the server
     * @return a ServerConfiguration representation of the server's informations
     * @throws MBeanException
     *             an error occured when communicating with server
     * @throws IOException
     *             a communication error occured
     */
    @SuppressWarnings("unchecked")
    public ServerConfiguration getServerConfiguration(String serverName)
        throws MBeanException, IOException {
        MBeanServerConnection connection = serverConf.getConnection();
        ServerConfiguration desiredServerConf = null;
        try {
            ObjectName serversListName = new ObjectName(connection
                .getDefaultDomain()
                + SERVERS_CONF_MBEAN_NAME);
            desiredServerConf = new ServerConfiguration(
                (HashMap<String, String>) connection.invoke(serversListName,
                    "getServerConfiguration", new Object[] {serverName},
                    new String[] {CLASS_STRING}));
        } catch (MalformedObjectNameException e) {
            throw new MBeanException(e);
        } catch (InstanceNotFoundException e) {
            throw new MBeanException(e);
        } catch (MBeanException e) {
            throw new MBeanException(e);
        } catch (ReflectionException e) {
            throw new MBeanException(e);
        }

        return desiredServerConf;
    }
    /**
     * Temporary method to get all service assemblies
     * 
     */
    public Map<String,JBIEntity> getServicesAssemblies(String serverName)
            throws MBeanException {
        PetalsJMXClient petalsJMXClient;
        try {
            
            if (!jmxUtils.storageServerReachable(serverConf)) {
                throw new MBeanException(null, "Server " + serverConf.getName()
                    + " not reachable");
            }
            ServerConfiguration serverConf=getServerConfiguration(serverName);
            String host = serverConf.getHost();
            int port = Integer.parseInt(serverConf.getJmxPort());

            petalsJMXClient = new PetalsJMXClient(host, port, "", "");

            PetalsDeploymentServiceClient pdsc = petalsJMXClient
                    .getDeploymentServiceClient();

            Map<String, JBIEntity> sas = new HashMap<String, JBIEntity>();
            
            for (String s :pdsc.getDeployedServiceAssemblies()) {
                JBIEntity sa = new JBIEntity();
                sa.setType(JBIEntity.SERVICE_ASSEMBLY);
                sa.setName(s);
                sa.setState(pdsc.getState(s));
                sas.put(s, sa);
                System.out.println("found SA :" +s +" - " + sa.getState());
            }
            return sas;

        } catch (ConnectionErrorException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (DeploymentServiceDoesNotExistException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (DeploymentServiceErrorException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (PerformActionErrorException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        } catch (IOException e) {
            e.printStackTrace();
            throw new MBeanException(e);
        }
    }
    
    /**
     * Gets the list of all servers known on the domain
     * 
     * @return a List representation of the servers
     * @throws IOException
     *             a communication error occured
     * @throws MBeanException
     *             the server throws an exception when communicating with it
     */
    @SuppressWarnings("unchecked")
    public List<ServerConfiguration> getServersList() throws MBeanException,
        IOException {
        // if ServerConfiguration has no connection, try to opens a new one
        // if unable to open one -> throws the IOException received
        if (serverConf.getConnection() == null) {
            serverConf.createConnection();
        }
        MBeanServerConnection connection = serverConf.getConnection();
        try {
            ObjectName serversListName = new ObjectName(connection
                .getDefaultDomain()
                + SERVERS_CONF_MBEAN_NAME);
            List<Map<String, String>> serversList = (List<Map<String, String>>) connection
                .getAttribute(serversListName, "Servers");
            List<ServerConfiguration> serversConf = new ArrayList<ServerConfiguration>();
            for (Map<String, String> currentServer : serversList) {
                serversConf.add(new ServerConfiguration(currentServer));
            }
            return serversConf;
        } catch (MalformedObjectNameException e) {
            throw new MBeanException(e);
        } catch (AttributeNotFoundException e) {
            throw new MBeanException(e);
        } catch (InstanceNotFoundException e) {
            throw new MBeanException(e);
        } catch (MBeanException e) {
            throw new MBeanException(e);
        } catch (ReflectionException e) {
            throw new MBeanException(e);
        }
    }

    /**
     * Gets the list of all servers known on the domain
     * 
     * @return a List representation of the servers
     * @throws IOException
     *             a communication error occured
     * @throws MBeanException
     *             the server throws an exception when communicating with it
     */
    @SuppressWarnings("unchecked")
    public List<Map<String, String>> getRawServersList() throws MBeanException,
        IOException {
        // if ServerConfiguration has no connection, try to opens a new one
        // if unable to open one -> throws the IOException received
        if (serverConf.getConnection() == null) {
            serverConf.createConnection();
        }
        MBeanServerConnection connection = serverConf.getConnection();
        try {
            ObjectName serversListName = new ObjectName(connection
                .getDefaultDomain()
                + SERVERS_CONF_MBEAN_NAME);
            List<Map<String, String>> serversList = (List<Map<String, String>>) connection
                .getAttribute(serversListName, "Servers");
            return serversList;
        } catch (MalformedObjectNameException e) {
            throw new MBeanException(e);
        } catch (AttributeNotFoundException e) {
            throw new MBeanException(e);
        } catch (InstanceNotFoundException e) {
            throw new MBeanException(e);
        } catch (MBeanException e) {
            throw new MBeanException(e);
        } catch (ReflectionException e) {
            throw new MBeanException(e);
        }
    }

    /**
     * Gets only started servers on domain
     * 
     * @return a List representation of ServerConfiguration of each server
     * @throws IOException
     * @throws MBeanException
     */
    public List<ServerConfiguration> getStartedServersList()
        throws IOException, MBeanException {
        List<ServerConfiguration> startedServers = new ArrayList<ServerConfiguration>();
        for (ServerConfiguration server : this.getServersList()) {
            if (server.getState().equalsIgnoreCase(PetalsAdminServiceMBean.STATE_STARTED)) {
                startedServers.add(server);
            }
        }
        return startedServers;
    }

    /**
     * Gets a Map of time datas from an MBean
     * 
     * @param serverConf
     *            the storage server where MBean is registered
     * @param mBeanName
     *            an Objetname representation of the MBean's name
     * @param mBeanType
     *            a String representation of the MBean's type
     * @param attributeName
     *            the attribute of the MBean containing the datas to retrieve
     * @param sinceTime
     *            the timestamp of beginning of the retrieval period
     * @return a Map representation of datas as <timestamp,number> pairs
     * @throws IOException
     * @throws MBeanException
     */
    @SuppressWarnings("unchecked")
    public Map<Long, Long> getTimeDatas(String mBeanName, String mBeanType,
        String datasetName, long sinceTime) throws IOException, MBeanException {

        // if ServerConfiguration has no connection, try to opens a new one
        // if unable to open one -> throws the IOException received
        if (serverConf.getConnection() == null) {
            serverConf.createConnection();
        }
        MBeanServerConnection connection = serverConf.getConnection();
        Map<Long, Long> timeDatas;
        try {
            ObjectName mBeanObjName = this.createObjectName(MBeanServerFactory
                .createMBeanServer(connection.getDefaultDomain()), mBeanName,
                mBeanType);
            timeDatas = (Map<Long, Long>) connection.invoke(mBeanObjName,
                "getDatas", new Object[] {datasetName, sinceTime},
                new String[] {CLASS_STRING, "long"});
        } catch (InstanceNotFoundException e) {
            throw new MBeanException(e);
        } catch (MBeanException e) {
            throw new MBeanException(e);
        } catch (ReflectionException e) {
            throw new MBeanException(e);
        } catch (MalformedObjectNameException e) {
            throw new MBeanException(e);
        }

        return timeDatas;

    }

    private ObjectName createObjectName(MBeanServer jmxServer,
        String mBeanName, String type) throws MalformedObjectNameException {
        return new ObjectName(jmxServer.getDefaultDomain() + ":name="
            + mBeanName + ",type=" + type);
    }

    /**
     * Return the configuration of the server with which the server communicate
     * 
     * @return a ServerConfiguration of a server
     */
    public ServerConfiguration getServerConf() {
        return serverConf;
    }

}
