/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JBIJMXConnectorUtil.java ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.webadmin.util;

import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

public final class JBIJMXConnectorUtil {

    private static final String ADMIN_NAME = "Admin";

    private static final String DEPLOYMENT_NAME = "Deployment";

    private static final String INSTALLATION_NAME = "Installation";

    private static final String NAME = "name";

    private static final String PETALS_DOMAIN = "Petals";

    private static final String SERVICE_TYPE = "service";

    private static final String TYPE = "type";

    // object for dates formating
    private static final DateFormat DATE_FORMATER = new SimpleDateFormat(
        "dd MMM yyyy, HH:mm:ss");

    private JBIJMXConnectorUtil() {
        // Do nothing
    }

    /**
     * Retrieve the object name of the admin service
     * 
     * @param mBeanServer
     *            mbean server
     * @return object name
     * @throws IOException
     * @throws NullPointerException
     * @throws MalformedObjectNameException
     */
    public static ObjectName getAdminServiceMBeanName(
        MBeanServerConnection mBeanServer) throws IOException,
        MalformedObjectNameException {
        ObjectName result = null;
        Set objNames;
        Map<String, String> attributes = new HashMap<String, String>();
        attributes.put(NAME, ADMIN_NAME);
        attributes.put(TYPE, SERVICE_TYPE);
        ObjectName objName = new ObjectName(PETALS_DOMAIN,
            new Hashtable<String, String>(attributes));
        objNames = mBeanServer.queryNames(objName, null);
        if (objNames != null && objNames.size() == 1) {
            result = (ObjectName) objNames.iterator().next();
        }
        return result;
    }

    /**
     * Retrieve the object name of a component
     * 
     * @param mBeanServer
     *            mbean server
     * @return object name
     * @throws IOException
     * @throws NullPointerException
     * @throws MalformedObjectNameException
     */
    public static ObjectName getComponentMBeanName(
        MBeanServerConnection mBeanServer, String name) throws IOException,
        MalformedObjectNameException {
        ObjectName result = null;
        String pattern = "*:type=engine,name=" + name;
        Set objNames;
        ObjectName objName = new ObjectName(pattern);
        objNames = mBeanServer.queryNames(objName, null);
        if (objNames != null && objNames.size() == 1) {
            result = (ObjectName) objNames.iterator().next();
        }
        if (result == null) {
            pattern = "*:type=binding,name=" + name;
            objName = new ObjectName(pattern);
            objNames = mBeanServer.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                result = (ObjectName) objNames.iterator().next();
            }
        }
        return result;
    }

    /**
     * Returns a JMX connector
     * 
     * @param host
     *            host
     * @param port
     *            port
     * @param username
     *            username
     * @param password
     *            password
     * @return jmx connector
     * @throws IOException
     */
    public static JMXConnector getConnection(String host, String port,
        String username, String password) throws IOException {
        JMXConnector connector = null;
        Properties env = System.getProperties();
        env.put("java.naming.factory.initial",
            "com.sun.jndi.rmi.registry.RegistryContextFactory");
        Map<String, Object> args = new HashMap<String, Object>();
        if (!"".equalsIgnoreCase(username)) {
            String passwd = password;
            if (password == null) {
                passwd = "";
            }
            String[] credentials = new String[] {username, passwd};
            args.put("jmx.remote.credentials", credentials);
        }
        String urlStr = "service:jmx:rmi:///jndi/rmi://" + host + ":" + port
            + "/management/rmi-jmx-connector";
        // log("INFO", "Trying to connect to the JMXConnector at :" + urlStr);

        JMXServiceURL url = new JMXServiceURL(urlStr);
        connector = JMXConnectorFactory.connect(url, args);
        return connector;
    }

    /**
     * Retrieve the object name of the deployment service
     * 
     * @param mBeanServer
     *            mbean server
     * @return object name
     * @throws IOException
     * @throws NullPointerException
     * @throws MalformedObjectNameException
     */
    public static ObjectName getDeploymentServiceMBeanName(
        MBeanServerConnection mBeanServer) throws IOException,
        MalformedObjectNameException {
        ObjectName result = null;
        Set objNames;
        Map<String, String> attributes = new HashMap<String, String>();
        attributes.put(NAME, DEPLOYMENT_NAME);
        attributes.put(TYPE, SERVICE_TYPE);
        ObjectName objName = new ObjectName(PETALS_DOMAIN,
            new Hashtable<String, String>(attributes));
        objNames = mBeanServer.queryNames(objName, null);
        if (objNames != null && objNames.size() == 1) {
            result = (ObjectName) objNames.iterator().next();
        }
        return result;
    }

    /**
     * Retrieve the object name of the installation service
     * 
     * @param mBeanServer
     *            mbean server
     * @return object name
     * @throws IOException
     * @throws NullPointerException
     * @throws MalformedObjectNameException
     */
    public static ObjectName getInstallationServiceMBeanName(
        MBeanServerConnection mBeanServer) throws IOException,
        MalformedObjectNameException {
        ObjectName result = null;
        Set objNames;
        Map<String, String> attributes = new HashMap<String, String>();
        attributes.put(NAME, INSTALLATION_NAME);
        attributes.put(TYPE, SERVICE_TYPE);
        ObjectName objName = new ObjectName(PETALS_DOMAIN,
            new Hashtable<String, String>(attributes));
        objNames = mBeanServer.queryNames(objName, null);
        if (objNames != null && objNames.size() == 1) {
            result = (ObjectName) objNames.iterator().next();
        }
        return result;
    }

    /**
     * Prints out a message on the system printstream.
     * 
     * @param level
     *            a user-defined level like INFO, WARNING, ERROR, etc.
     * @param message
     *            the message to print
     */
    @SuppressWarnings("unused")
    private static void log(String level, String message) {
        System.out.println("[WEBADMIN " + level + "] "
            + DATE_FORMATER.format(new Date().getTime()) + " : " + message);
    }

}
