/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JMXUtils.java msauvage $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.webadmin.util;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.management.remote.JMXConnector;

import org.objectweb.petals.kernel.admin.PetalsAdminServiceMBean;
import org.objectweb.petals.tools.webadmin.bo.ServerConfiguration;

public final class JMXUtils {

    private static final String PETALS_ADMIN_NAME = "Petals:name=PetalsAdmin,type=service";

    private static final String PETALS_MONITORING_NAME = "Petals:name=Monitoring,type=service";

    private static final String ALL = "ALL";

    /**
     * The single JMXUtils object
     */
    private static JMXUtils jmxUtils;

    private static final String DOMAIN_NAME = "domainName";

    private static final String CONTAINER_NAME = "containerName";

    /**
     * Constructs a new empty JMXUtils
     */
    private JMXUtils() {
    }

    /**
     * returns the singleton JMXUtils object
     * 
     * @return
     */
    public static JMXUtils getJMXUtils() {
        if (null == jmxUtils) {
            jmxUtils = new JMXUtils();
        }
        return jmxUtils;
    }

    /**
     * Returns all servers of the domain, started and stopped, via a JMX call to
     * a known server
     * 
     * @param serverConf
     *            the ServerConfiguration of a server known on the domain
     * @return list of server's configurations
     * @throws IOException
     *             PEtALS server to contact is unreachable
     * @throws MBeanException
     *             a communication occured when getting servers
     */
    @SuppressWarnings("unchecked")
    public List<ServerConfiguration> getAllServers(
        ServerConfiguration serverConf) throws IOException, MBeanException, AttributeNotFoundException {
        if (!this.petalsServerReachable(serverConf)) {
            throw new IOException("Entry server " + serverConf.getName()
                + " unreachable");
        }
        List<ServerConfiguration> serversList = new ArrayList<ServerConfiguration>();

        MBeanServerConnection connection = serverConf.getConnection();
        Collection<Map<String, String>> serversConfigurations = new ArrayList<Map<String, String>>();
        try {
            ObjectName petalsAdminName = new ObjectName(PETALS_ADMIN_NAME);
            serversConfigurations = (Set<Map<String, String>>) connection
                .invoke(petalsAdminName, "retrieveContainersConfiguration", new Object[]{}, new String[]{});
        } catch (InstanceNotFoundException e) {
            throw new MBeanException(e);
        } catch (MBeanException e) {
            throw new MBeanException(e);
        } catch (ReflectionException e) {
            throw new MBeanException(e);
        } catch (MalformedObjectNameException e) {
            e.printStackTrace(System.err);
            // should not occure : name is well formed...
        }
        for (Map<String, String> currentServer : serversConfigurations) {
            if (currentServer.containsKey((DOMAIN_NAME))) {
                continue;
            }
            ServerConfiguration currentServerConf = new ServerConfiguration(
                currentServer.get(CONTAINER_NAME), currentServer.get(PetalsAdminServiceMBean.CONF_HOST),
                currentServer.get(PetalsAdminServiceMBean.CONF_JMX_RMI_PORT), currentServer.get(PetalsAdminServiceMBean.CONF_JMX_LOGIN),
                currentServer.get("jmxPassword"), currentServer.get(PetalsAdminServiceMBean.CONF_STATE));
            if (currentServerConf.getState().equalsIgnoreCase(PetalsAdminServiceMBean.STATE_STARTED)) {
                // creation of a connection for the server if it's started
                MBeanServerConnection currentServerConnection = this
                    .retrieveConnector(currentServerConf)
                    .getMBeanServerConnection();
                currentServerConf.setConnection(currentServerConnection);
            }
            serversList.add(currentServerConf);
        }
        return serversList;
    }

    /**
     * Returns a server's name, based on his jmx informations
     * 
     * @param serverConf
     *            the configuration of the server
     * @return the name of the server, null if not found
     * @throws IOException
     * @throws MBeanException
     * @throws AttributeNotFoundException 
     */
    public String getServerName(ServerConfiguration serverConf)
            throws MBeanException, IOException, AttributeNotFoundException {
        List<ServerConfiguration> serversList = this.getAllServers(serverConf);
        for (ServerConfiguration currentServer : serversList) {
            if (currentServer.getHost().equalsIgnoreCase(serverConf.getHost())
                    && currentServer.getJmxPort().equalsIgnoreCase(
                            serverConf.getJmxPort())) {
                return currentServer.getName();
            }
        }
        return null;
    }

    /**
     * Retrieve all servers actually started on a domain via a JMX call to a
     * server known.
     * 
     * @param serverConf
     *            the ServerConfiguration of a server known on the domain
     * @return list of server's configurations
     * @throws IOException
     *             PEtALS server to contact is unreachable
     * @throws MBeanException
     *             a communication occured when getting servers
     * @throws AttributeNotFoundException 
     */
    public List<ServerConfiguration> getStartedServers(
            ServerConfiguration serverConf) throws IOException, MBeanException, AttributeNotFoundException {
        // list of the servers name to return
        List<ServerConfiguration> serversList = this.getAllServers(serverConf);
        // list of server's index in servers list to remove
        List<Integer> toRemove = new ArrayList<Integer>();
        // search for servers to remove -> adding them index to remove list
        for (ServerConfiguration currentServer : serversList) {
            if (currentServer.getState().equalsIgnoreCase(PetalsAdminServiceMBean.STATE_STOPPED)) {
                toRemove.add(new Integer(serversList.indexOf(currentServer)));
            }
        }
        // removing servers
        for (Integer serverIndex : toRemove) {
            serversList.remove(serverIndex.intValue());
        }
        return serversList;
    }

    /**
     * Return all components installed on a server. Connection must be non null
     * and server must be accessible via this connection.
     * 
     * @param connection
     *            a MBeanServerConnection to the server
     * @return a Map of JBIEntity components as <name,component>
     * @throws MBeanException
     */
    public Map<String, JBIEntity> listComponents(
            MBeanServerConnection connection) throws MBeanException {
        return this.listComponents(connection, ALL);
    }

    /**
     * Lists the deployed JBI components of a specified type. Connection must be
     * non null and server must be accessible via this connection.
     * 
     * @param connection
     *            a MBeanServerConnection to the server
     * @param type
     *            type of component : BC for Binding Components, SE for Service
     *            Engine, ALL for both BC and SE
     * @return a Map of JBIEntity components as <name,component>
     * @throws MBeanException
     * @throws Exception
     */
    public Map<String, JBIEntity> listComponents(
            MBeanServerConnection connection, String type)
            throws MBeanException {
        Map<String, JBIEntity> components = new HashMap<String, JBIEntity>();
        if (type.equalsIgnoreCase("BC") || type.equalsIgnoreCase(ALL)) {
            components.putAll(this.getComponents(connection,
                    "BindingComponents", JBIEntity.BINDING_COMPONENT));
        }
        if (type.equalsIgnoreCase("SE") || type.equalsIgnoreCase(ALL)) {
            components.putAll(this.getComponents(connection,
                    "EngineComponents", JBIEntity.SERVICE_ENGINE));
        }
        return components;
    }

    /**
     * Retrieve a JMX connector for a given server
     * 
     * @return a JMXConnector
     * @throws IOException
     */
    public JMXConnector retrieveConnector(ServerConfiguration serverConf)
            throws IOException {
        return JBIJMXConnectorUtil.getConnection(serverConf.getHost(),
                serverConf.getJmxPort(), serverConf.getJmxLogin(), serverConf
                        .getJmxPassword());
    }

    /**
     * Retrieve a JMX connector for a server with connection parameters given
     * 
     * @return a JMXConnector
     * @throws IOException
     */
    public JMXConnector retrieveConnector(String host, String jmxPort,
            String jmxLogin, String jmxPassword) throws IOException {
        return JBIJMXConnectorUtil.getConnection(host, jmxPort, jmxLogin,
                jmxPassword);
    }

    /**
     * Indicates if a PEtALS server is accessible via JMX or not by calling a
     * getAttribute on the administration MBean via his connection attribute
     * 
     * @param serverConf
     *            the server's configuration of the server to test
     * @return true if server responds, false if not or if an exception occured
     */
    public boolean petalsServerReachable(ServerConfiguration serverConf) {
        boolean reachable = true;
        try {
            if (serverConf.getConnection() != null) {
                ObjectName petalsMonitoringName = new ObjectName(
                        PETALS_MONITORING_NAME);
                // response has no interest -> if server replies, it is
                // reachable
                serverConf.getConnection().getAttribute(petalsMonitoringName,
                        "MessagesThreshold");
            } else {
                reachable = false;
            }
        } catch (MalformedObjectNameException e) {
            reachable = false;
        } catch (InstanceNotFoundException e) {
            reachable = false;
        } catch (MBeanException e) {
            reachable = false;
        } catch (IOException e) {
            reachable = false;
        }
        // errors throws by the server -> there is communication, it is
        // reachable !
        catch (ReflectionException e) {
            reachable = true;
        } catch (AttributeNotFoundException e) {
            reachable = true;
        }
        return reachable;
    }

    /**
     * Indicates if a storage server is reachable by calling a getAttribute on
     * its administration MBean via his connection attribute
     * 
     * @param serverConf
     *            the configuration of the server to test
     * @return true if server replies correctly, false if not or if an exception
     *         occured
     */
    public boolean storageServerReachable(ServerConfiguration serverConf) {
        boolean reachable = true;
        try {
            if (serverConf.getConnection() != null) {
                ObjectName petalsAdminName = new ObjectName(serverConf
                        .getConnection().getDefaultDomain()
                        + ":name=StorageAdmin,type=service");
                serverConf.getConnection()
                        .getAttribute(petalsAdminName, "Ping");
            } else {
                reachable = false;
            }
        } catch (MalformedObjectNameException e) {
            reachable = false;
        } catch (AttributeNotFoundException e) {
            reachable = false;
        } catch (InstanceNotFoundException e) {
            reachable = false;
        } catch (MBeanException e) {
            reachable = false;
        } catch (ReflectionException e) {
            reachable = false;
        } catch (IOException e) {
            reachable = false;
        }
        return reachable;
    }

    /**
     * Gets components of a specific type installed on a PEtALS server via a JMX
     * call to the administration MBean. All parameters must be given.
     * 
     * @param connection
     *            connection to the server
     * @param typeName
     *            the name of the type of components searched, as registered on
     *            server
     * @param typeLabel
     *            the label of the type to give to the component
     * @return a Map of JBIEntity representations of components
     * @throws MBeanException
     *             a problem occured in JMX call
     */
    private Map<String, JBIEntity> getComponents(
            MBeanServerConnection connection, String typeName, String typeLabel)
            throws MBeanException {
        Map<String, JBIEntity> components = new HashMap<String, JBIEntity>();
        Object result;
        try {
            if ("BindingComponents".equalsIgnoreCase(typeName)) {
                result= connection.invoke(JBIJMXConnectorUtil
                        .getAdminServiceMBeanName(connection), "getBindingComponents",
                        new String[] {}, new String[] {});  
            }
            else {
             result= connection.invoke(JBIJMXConnectorUtil
                     .getAdminServiceMBeanName(connection), "getEngineComponents",
                     new String[] {}, new String[] {});
            }
            if (result instanceof ObjectName[]) {
                ObjectName[] engines = (ObjectName[]) result;
                for (ObjectName objectName : engines) {
                    JBIEntity component = new JBIEntity();
                    String objState = (String) connection.getAttribute(
                            objectName, "CurrentState");
                    String name = objectName.getKeyProperty("name");
                    component.setName(name);
                    component.setState(objState);
                    component.setObjectName(objectName);
                    component.setDescription("");
                    component.setType(typeLabel);
                    components.put(component.getName(), component);
                }
            }
        } catch (AttributeNotFoundException e) {
            throw new MBeanException(e);
        } catch (InstanceNotFoundException e) {
            throw new MBeanException(e);
        } catch (MalformedObjectNameException e) {
            throw new MBeanException(e);
        } catch (MBeanException e) {
            throw new MBeanException(e);
        } catch (ReflectionException e) {
            throw new MBeanException(e);
        } catch (IOException e) {
            throw new MBeanException(e);
        }
        return components;
    }
}
