/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServerPropertiesManager.java msauvage $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.webadmin.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Map;
import java.util.Properties;

/**
 * Manage a properties file.
 *
 * @author msauvage chamerling - eBMWebSourcing
 *
 */
public class ServerPropertiesManager {

    /**
     * The default properties file name
     */
    private static final String PROPERTIES_FILE_NAME = "entryServer.properties";

    /**
     * The storage object
     */
    private Properties properties;

    /**
     * The root path of the properties file.
     */
    private String rootPath;

    /**
     * Creates a new instance of {@link ServerPropertiesManager}
     *
     */
    public ServerPropertiesManager() {
        this.properties = new Properties();
    }

    /**
     * Creates a new instance of {@link ServerPropertiesManager} with a path for
     * the file associated
     *
     */
    public ServerPropertiesManager(String rootPath) {
        this.properties = new Properties();
        this.rootPath = rootPath;
    }

    /**
     * Gets the file name where properties are stored
     *
     * @return a String representation of the name
     */
    public String getPropertiesFileName() {
        return PROPERTIES_FILE_NAME;
    }

    /**
     *
     *
     */
    public void clear() {
        this.properties = new Properties();
    }

    /**
     * Returns the value corresponding to the key in the properties as a String.
     *
     * @param key
     *            the key searched
     * @return a String representation of the value corresponding to the key,
     *         null if key doesn't exists
     */
    public String getAttributeValue(String key) {
        return properties.getProperty(key);
    }

    /**
     * Returns all the properties
     *
     * @return the properties as a Properties object
     */
    public Properties getProperties() {
        return this.properties;
    }

    public String getRootPath() {
        return rootPath;
    }

    /**
     * Load properties from file.
     *
     * @return boolean true if file is correctly loaded, false if file is not
     *         found
     * @throws IOException
     */
    public boolean load() throws IOException {
        File f = null;
        boolean result = true;

        if (rootPath != null) {
            f = new File(rootPath, PROPERTIES_FILE_NAME);
            if (!f.exists()) {
                throw new FileNotFoundException("Properties file not found on "
                    + f.getAbsolutePath());
            }

            try {
                // load properties
                FileInputStream fis = new FileInputStream(f);
                properties.load(fis);
                fis.close();
            } catch (IOException e) {
                throw new IOException(
                    "Permission Denied : Properties file is not readable at "
                        + f.getAbsolutePath());
            }
        } else {
            throw new FileNotFoundException(
                "No root path setted for properties file");
        }
        return result;
    }

    /**
     * Save the actual properties into a file without comment
     *
     * @throws IOException
     */
    public void save() throws IOException {
        this.save("");
    }

    /**
     * Save the actual properties into a file with a comment
     *
     * @param file
     *            the complete path to the file to write
     */
    public void save(String comments) throws IOException {
        FileOutputStream os = null;
        File f = new File(rootPath, PROPERTIES_FILE_NAME);
        try {
            os = new FileOutputStream(f);
            properties.store(os, comments);
            os.close();
        } catch (IOException e) {
            throw new IOException(
                "Permission Denied : Properties file is not writable at "
                    + f.getAbsolutePath());
        }
    }

    /**
     * Sets a new key/value property. If property already exist, value is
     * updated
     *
     * @param key
     *            the key of the property
     * @param value
     *            the value corresponding
     */
    public void setAttribute(String key, String value) {
        if (value == null) {
            properties.remove(key);
        } else {
            properties.setProperty(key, value);
        }
    }

    /**
     * Copies all of the mappings from the specified Map to the properties
     * attribute.
     *
     * @param properties
     *            a Map of the <key,value> pairs to be stored
     */
    public void setAttributes(Map<String, String> attributes) {
        if (attributes != null) {
            this.properties.putAll(attributes);
        }
    }

    public void setProperties(Properties properties) {
        this.properties = properties;
    }

    /**
     *
     * @param rootPath
     */
    public final void setRootPath(String rootPath) {
        this.rootPath = rootPath;
    }

}
